
from singer.catalog import Catalog, Schema, CatalogEntry
from singer import metadata
from . import schemas


def load_schema(ctx, tap_stream_id):
    schema = getattr(schemas, tap_stream_id)
    return schema


def discover(ctx):
    catalog = Catalog([])
    for stream in all_streams:
        schema = Schema.from_dict(load_schema(ctx, stream.tap_stream_id), inclusion="automatic")
        mdata = metadata.new()
        mdata = metadata.write(mdata, (), 'selected-by-default', True)
        catalog.streams.append(CatalogEntry(
            stream=stream.tap_stream_id,
            tap_stream_id=stream.tap_stream_id,
            key_properties=stream.pk_fields,
            schema=schema,
            metadata=metadata.to_list(mdata)
        ))
    return catalog


class Stream(object):
    def __init__(self, tap_stream_id, pk_fields):
        self.tap_stream_id = tap_stream_id
        self.pk_fields = pk_fields


all_streams = [
    Stream("historical_adherence", ['userId', 'management_unit_id', 'startDate']),
    Stream("users", ['id']),
    Stream("groups", ['id']),
    Stream("location", ['id']),
    Stream("conversation", ['conversation_id']),
    Stream("user_state", ['id']),
    Stream("management_unit", ['id']),
    Stream("activity_codes", ['id', 'management_unit_id']),
    Stream("management_unit_users", ['user_id', 'management_unit_id']),
    Stream("user_schedule", ['start_date', 'user_id']),
    Stream("presence", ['id']),
    Stream("queues", ['id']),
    Stream("queue_membership", ['id']),
    Stream("queue_wrapup_code", ['id']),
]



import singer
from singer import bookmarks as bks_


class Context(object):
    """Represents a collection of global objects necessary for performing
    discovery or for running syncs. Notably, it contains

    - state   - The mutable state dict that is shared among streams
    """
    def __init__(self, state):
        self.state = state

    def get_bookmark(self, path):
        return bks_.get_bookmark(self.state, *path)

    def set_bookmark(self, path, val):
        bks_.write_bookmark(self.state, path[0], path[1], val)

    def get_offset(self, path):
        off = bks_.get_offset(self.state, path[0])
        return (off or {}).get(path[1])

    def set_offset(self, path, val):
        bks_.set_offset(self.state, path[0], path[1], val)

    def clear_offsets(self, tap_stream_id):
        bks_.clear_offset(self.state, tap_stream_id)

    def write_state(self):
        singer.write_state(self.state)
